
# q-expansion of newform 31.3.b.b, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 31
weight = 3
poly_data = [-1, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-4, 0, 1], 1]]

hecke_ring_character_values = [[3, [-1, 0, 0]]]
aps_data = [[0, 0, 1], [0, 0, 0], [0, -3, -1], [0, 5, -1], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, -11, 7], [0, 0, 0], [0, 0, 0], [-31, 0, 0], [0, 0, 0], [0, 21, -17], [0, 0, 0], [-30, 0, 0], [0, 0, 0], [0, 21, 23], [0, 0, 0], [10, 0, 0], [0, -51, 7], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 5, -49], [0, 69, 7], [0, -35, -41], [0, 45, -49], [0, 29, 47], [0, -75, 31], [0, 0, 0], [138, 0, 0], [0, 0, 0], [0, 0, 0], [-198, 0, 0], [0, 0, 0], [0, -35, 79], [0, -115, -1], [0, 0, 0], [-150, 0, 0], [0, 0, 0], [0, 0, 0], [0, 21, -97], [0, -35, -89], [0, 0, 0], [0, 0, 0], [0, -91, -73], [0, 0, 0], [330, 0, 0], [0, 0, 0], [0, 165, -1], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 45, 119], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 189, -73], [-550, 0, 0], [90, 0, 0], [0, 205, 31], [0, 141, 103], [0, 0, 0], [0, 45, -161], [0, 0, 0], [0, 0, 0], [0, 0, 0], [202, 0, 0], [0, 0, 0], [0, -171, -113], [0, 0, 0], [0, -235, 119], [-358, 0, 0], [0, 0, 0], [0, 0, 0], [0, 245, 71], [0, 0, 0], [0, 0, 0], [0, 261, -137], [0, 301, -17], [738, 0, 0], [0, 0, 0], [0, -59, 223], [0, -315, 71], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, -315, -41], [0, -99, -217], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 285, 119], [0, 0, 0], [-942, 0, 0], [0, 0, 0], [0, -371, 127], [0, -275, -161], [0, 0, 0], [0, -195, 271], [0, 0, 0], [0, 0, 0], [-830, 0, 0], [0, 0, 0], [0, -315, 239], [0, 69, 287], [0, 0, 0], [1090, 0, 0], [0, 0, 0], [-750, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [810, 0, 0], [0, 429, -193], [0, -91, -313], [0, 0, 0], [0, 0, 0], [0, -195, -289], [0, 221, -337], [0, -219, 343], [0, 0, 0], [0, 0, 0], [0, -355, 311], [0, 325, -329], [0, 0, 0], [0, 0, 0], [0, 469, 127], [0, 0, 0], [0, 0, 0], [0, -539, -17], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [-1478, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [-1422, 0, 0], [1210, 0, 0], [-270, 0, 0], [0, 0, 0], [0, 0, 0], [0, -395, 391], [0, 0, 0], [0, 0, 0], [0, 165, -449], [0, 85, 439], [0, 0, 0], [-1262, 0, 0], [0, 0, 0], [-110, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0], [-1158, 0, 0], [0, -435, -329], [0, 0, 0], [0, 0, 0], [0, -715, 119]]
